#pragma once
#include <PSX/AutoLBA/auto_lba.hpp>
#include <PSX/Auxiliary/math_helper.hpp>
#include <PSX/System/IOPorts/cd_io.hpp>
#include <stddef.hpp>

namespace JabyEngine {
    namespace CD {
        namespace internal {
            enum struct State {
                Free = 0,
                Done = 0,

                Reading,
                BufferFull,
                Error,
            };

            class SectorBufferAllocator {
            private:
                typedef CD_IO::DataSector* (*AllocatorFunction)(void* ctx);

            private:
                void*             ctx      = nullptr;
                AllocatorFunction allocate = nullptr;

                constexpr SectorBufferAllocator(void* ctx, AllocatorFunction function) : ctx(ctx), allocate(function) {
                }

            public:
                constexpr SectorBufferAllocator() = default;

                static constexpr SectorBufferAllocator create(void* obj, AllocatorFunction function) {
                    return SectorBufferAllocator(obj, function);
                }

                static constexpr SectorBufferAllocator invalid() {
                    return SectorBufferAllocator(nullptr, nullptr);
                }

                inline CD_IO::DataSector* allocate_sector() const {
                    return this->allocate(this->ctx);
                }
            };

            struct CDTimeStamp {
                static constexpr size_t MaxSector  = 75;
                static constexpr size_t MaxSeconds = 60;

                uint8_t min;
                uint8_t sec;
                uint8_t sector;

                static constexpr CDTimeStamp from(uint32_t lba) {
                    const auto [min, new_lba] = div_and_mod(lba, MaxSector*MaxSeconds);
                    const auto [sec, sectors] = div_and_mod(new_lba, MaxSector);

                    return CDTimeStamp{static_cast<uint8_t>(min), static_cast<uint8_t>(sec), static_cast<uint8_t>(sectors)};
                }

                constexpr uint8_t get_min_cd()    const {
                    return to_bcd(this->min);
                }

                constexpr uint8_t get_sec_cd()    const {
                    return to_bcd(this->sec);
                }

                constexpr uint8_t get_sector_cd() const {
                    return to_bcd(this->sector);
                }
            };
        }
    }
}