#include "../../internal-include/CD/cd_internal.hpp"
#include <PSX/Audio/CDDA.hpp>

namespace JabyEngine {
    namespace CDDA {
        namespace CD = JabyEngine::CD::internal;

        static CD::CDTimeStamp playing_track;
        static CD::CDTimeStamp last_track;

        TrackList get_tracks() {
            CD::Command::wait_completed();

            CD_IO::PortIndex0::change_to();
            CD::Command::send_wait<CD_IO::PortIndex0>(CD_IO::Command::GetTN);
            
            const auto stat       = CD_IO::PortIndex0::ResponseFifo.read();
            const auto first      = CD_IO::PortIndex0::ResponseFifo.read().raw;
            const auto end        = CD_IO::PortIndex0::ResponseFifo.read().raw;
            const auto last_track = (end - first) + 1;

            if(last_track == 1) {
                return TrackList::empty();
            }

            return TrackList{.first_track = 2, .last_track = static_cast<uint8_t>(last_track)};
        }

        void play(uint8_t track) {
            CD::enable_CDDA(); // < Command waits

            CD::Command::send_wait<CD_IO::PortIndex0>(CD_IO::Command::GetTD, track);
            const auto stats  = CD_IO::PortIndex0::ResponseFifo.read().raw;
            playing_track.min = CD_IO::PortIndex0::ResponseFifo.read().raw;
            playing_track.sec = CD_IO::PortIndex0::ResponseFifo.read().raw;

            CD::Command::send<CD_IO::PortIndex0>(CD_IO::Command::Play, track);
        }

        void stop() {
            CD::Command::wait_completed();
            CD::Command::send<CD_IO::PortIndex0>(CD_IO::Command::Pause);
        }

        void push_play() {
            stop();
            CD::Command::wait_completed();
            CD::Command::send_wait<CD_IO::PortIndex0>(CD_IO::Command::GetLocP);

            const auto track  = CD_IO::PortIndex0::ResponseFifo.read().raw; // track number (AAh=Lead-out area) (FFh=unknown, toc, none?)
            const auto index  = CD_IO::PortIndex0::ResponseFifo.read().raw; // index number (Usually 01h)
            const auto mm     = CD_IO::PortIndex0::ResponseFifo.read().raw; // minute number within track (00h and up)
            const auto ss     = CD_IO::PortIndex0::ResponseFifo.read().raw; // second number within track (00h to 59h)
            const auto sect   = CD_IO::PortIndex0::ResponseFifo.read().raw; // sector number within track (00h to 74h)
            last_track.min    = CD_IO::PortIndex0::ResponseFifo.read().raw; // minute number on entire disk (00h and up)
            last_track.sec    = CD_IO::PortIndex0::ResponseFifo.read().raw; // second number on entire disk (00h to 59h)
            last_track.sector = CD_IO::PortIndex0::ResponseFifo.read().raw; // sector number on entire disk (00h to 74h)
        }

        void pop_play() {
            CD::Command::wait_completed();
            CD::enable_CDDA(); // < Command waits
            CD::Command::send_wait<CD_IO::PortIndex0>(CD_IO::Command::SetLoc, last_track.min, last_track.sec, last_track.sector);
            CD::Command::send<CD_IO::PortIndex0>(CD_IO::Command::Play);
        }
    }
}